# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: mb_protocol_answers.proto
# Protobuf Python Version: 5.28.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    2,
    '',
    'mb_protocol_answers.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import wmbc.mb_proto.mb_protocol_enums_pb2 as mb__protocol__enums__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19mb_protocol_answers.proto\x12\x15wmb.proto.mb_protocol\x1a\x17mb_protocol_enums.proto\"C\n\x08\x41\x63kFrame\x12\x37\n\x0b\x61\x63knowladge\x18\x01 \x01(\x0e\x32\".wmb.proto.mb_protocol.Acknowladge\"0\n\x17PeriodicalConfiguration\x12\x15\n\rconfiguration\x18\x01 \x01(\r\"\xed\x05\n\x13\x44iagnosticsAnsFrame\x12\x18\n\x10\x66irmware_version\x18\x01 \x01(\r\x12\x11\n\tdevice_id\x18\x02 \x01(\r\x12<\n\x0etransport_type\x18\x03 \x01(\x0e\x32$.wmb.proto.mb_protocol.TransportType\x12\x18\n\x10last_reset_cause\x18\x04 \x01(\r\x12\x1a\n\x12last_fault_address\x18\x05 \x01(\r\x12\x36\n\x0b\x64\x65vice_mode\x18\x06 \x01(\x0e\x32!.wmb.proto.mb_protocol.ModbusMode\x12@\n\x10\x61ntenna_settings\x18\x07 \x01(\x0e\x32&.wmb.proto.mb_protocol.AntennaSettings\x12\x0e\n\x06uptime\x18\x08 \x01(\r\x12\x34\n\x0b\x62\x61ud_port_0\x18\t \x01(\x0e\x32\x1f.wmb.proto.mb_protocol.PortBaud\x12\x34\n\x0b\x62\x61ud_port_1\x18\n \x01(\x0e\x32\x1f.wmb.proto.mb_protocol.PortBaud\x12\x38\n\rparity_port_0\x18\x0b \x01(\x0e\x32!.wmb.proto.mb_protocol.PortParity\x12\x38\n\rparity_port_1\x18\x0c \x01(\x0e\x32!.wmb.proto.mb_protocol.PortParity\x12=\n\x10stop_bits_port_0\x18\r \x01(\x0e\x32#.wmb.proto.mb_protocol.PortStopBits\x12=\n\x10stop_bits_port_1\x18\x0e \x01(\x0e\x32#.wmb.proto.mb_protocol.PortStopBits\x12M\n\x15modbus_configurations\x18\x0f \x03(\x0b\x32..wmb.proto.mb_protocol.PeriodicalConfiguration\"\x80\x01\n\x13ModbusResponseFrame\x12\x36\n\x0bmodbus_port\x18\x01 \x01(\x0e\x32!.wmb.proto.mb_protocol.ModbusPort\x12\x1b\n\x13\x63onfiguration_index\x18\x02 \x01(\r\x12\x14\n\x0cmodbus_frame\x18\x03 \x01(\x0c\"\xed\x01\n\x0b\x41nswerFrame\x12\x34\n\tack_frame\x18\x01 \x01(\x0b\x32\x1f.wmb.proto.mb_protocol.AckFrameH\x00\x12K\n\x15\x64iagnostics_ans_frame\x18\x02 \x01(\x0b\x32*.wmb.proto.mb_protocol.DiagnosticsAnsFrameH\x00\x12K\n\x15modbus_response_frame\x18\x03 \x01(\x0b\x32*.wmb.proto.mb_protocol.ModbusResponseFrameH\x00\x42\x0e\n\x0c\x61nswer_frame*R\n\x0b\x41\x63knowladge\x12\x17\n\x13\x41\x43KNOWLADGE_UNKNOWN\x10\x00\x12\x14\n\x0f\x41\x43KNOWLADGE_ACK\x10\xaa\x01\x12\x14\n\x10\x41\x43KNOWLADGE_NACK\x10w*G\n\rTransportType\x12\x1a\n\x16TRANSPORT_TYPE_UNKNOWN\x10\x00\x12\x1a\n\x16TRANSPORT_TYPE_WIREPAS\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'mb_protocol_answers_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_ACKNOWLADGE']._serialized_start=1319
  _globals['_ACKNOWLADGE']._serialized_end=1401
  _globals['_TRANSPORTTYPE']._serialized_start=1403
  _globals['_TRANSPORTTYPE']._serialized_end=1474
  _globals['_ACKFRAME']._serialized_start=77
  _globals['_ACKFRAME']._serialized_end=144
  _globals['_PERIODICALCONFIGURATION']._serialized_start=146
  _globals['_PERIODICALCONFIGURATION']._serialized_end=194
  _globals['_DIAGNOSTICSANSFRAME']._serialized_start=197
  _globals['_DIAGNOSTICSANSFRAME']._serialized_end=946
  _globals['_MODBUSRESPONSEFRAME']._serialized_start=949
  _globals['_MODBUSRESPONSEFRAME']._serialized_end=1077
  _globals['_ANSWERFRAME']._serialized_start=1080
  _globals['_ANSWERFRAME']._serialized_end=1317
# @@protoc_insertion_point(module_scope)



import logging
import os
import sys
from time import time, sleep
import asyncio
import signal

from wsctrl.sink_ctrl import SinkController, Nbor
from wmbc.mb_proto import mb_protocol_enums_pb2 as mb_enums
from wmbc.mb_proto.mb_protocol_iface import MBProto
from google.protobuf.json_format import MessageToJson


logging.basicConfig(level=logging.INFO)
   

def signal_exit(sig, frame):
    sys.exit(0)

signal.signal(signal.SIGINT, signal_exit)


class WMBController():
    """
    Implementation of WMB controller
    """

    MB_PROTO_SRC_EP = 77
    MB_PROTO_DST_EP = 66

    def __init__(self, **kwargs):

        logging.info("Starting WMB Controller")
        self._sink_ids = kwargs.get("sink_ids")
        self._cmd_type = kwargs.get("cmd")
        self._dst_addr = kwargs.get("dst_addr")
        self._dev_mode = kwargs.get("dev_mode")
        self._ant_cfg = kwargs.get("ant_cfg")
        self._target_port = kwargs.get("target_port")
        self._port_cfg = kwargs.get("port_cfg")
        self._modbus_file = kwargs.get("modbus_file")
        self._modbus_frame = kwargs.get("modbus_frame")
        self._modbus_interval = kwargs.get("modbus_interval")
        self._modbus_cfg_idx = kwargs.get("modbus_cfg_idx")
        self._polling_only = self._cmd_type is None

        self._mbproto = MBProto()
        # Provide default sink_ids if not provided in init
        if (self._sink_ids is None):
            self._sink_ids = ["sink0", "sink1"]
        if (self._cmd_type is not None):
            assert self._dst_addr > 0, "Destination address cannot be 0!"
            self._client = SinkController(self._dst_addr & 0xFFFFFFFF, self.MB_PROTO_SRC_EP, self.MB_PROTO_DST_EP, sink_ids=self._sink_ids)
        else:
            self._client = SinkController(self._dst_addr, pm=True, sink_ids=self._sink_ids)
            return

        if self._cmd_type == "reset":
            self._payload_coded = self._mbproto.create_device_reset()
        elif self._cmd_type == "diag":
            self._payload_coded = self._mbproto.create_diagnostics()
        elif self._cmd_type == "dev_mode":
            self._mbproto.device_mode = self._dev_mode
            self._payload_coded = self._mbproto.create_device_mode()
        elif self._cmd_type == "ant_cfg":
            self._mbproto.antenna_config = self._ant_cfg
            self._payload_coded = self._mbproto.create_antenna_config()
        elif self._cmd_type == "port_cfg":
            self._mbproto.target_port = self._target_port
            self._mbproto.baudrate_config = int(self._port_cfg[0])
            self._mbproto.parity_bit = int(self._port_cfg[1])
            self._mbproto.stop_bits = int(self._port_cfg[2])
            self._payload_coded = self._mbproto.create_port_config()
        elif self._cmd_type == "modbus_1s":
            if (self._modbus_file is not None and self._modbus_frame is not None):
                raise ValueError("Both file and frame defined as payload")
            elif (self._modbus_file is not None or self._modbus_frame is not None):
                if (self._modbus_file is not None):
                    try:
                        with open(self._modbus_file, 'rb') as f:
                            self._payload_coded = f.read()
                    except FileNotFoundError:
                        raise FileNotFoundError(f"File {self._modbus_file} does not exist!")
                    except Exception as e:
                        raise e(f"Unhandled exception")
                    self._mbproto.target_port = self._target_port
                    self._payload_coded = self._mbproto.create_modbus_oneshot(self._payload_coded)
                else:
                    self._payload_coded = self._modbus_frame
                    self._mbproto.target_port = self._target_port
                    self._payload_coded = self._mbproto.create_modbus_oneshot(self._payload_coded)
            else:
                raise ValueError("No Modbus Payload!")
        elif self._cmd_type == "modbus_p":
            if (self._modbus_file is not None and self._modbus_frame is not None):
                raise ValueError("Both file and frame defined as payload")
            elif (self._modbus_file is not None or self._modbus_frame is not None):
                if (self._modbus_file is not None):
                    try:
                        with open(self._modbus_file, 'rb') as f:
                            self._payload_coded = f.read()
                    except FileNotFoundError:
                        raise FileNotFoundError(f"File {self._modbus_file} does not exist!")
                    except Exception as e:
                        raise e(f"Unhandled exception")
                    self._mbproto.target_port = self._target_port
                    self._payload_coded = self._mbproto.create_modbus_periodic(self._modbus_cfg_idx, self._modbus_interval,
                                                                               self._payload_coded)
                else:
                    self._payload_coded = self._modbus_frame
                    self._mbproto.target_port = self._target_port
                    self._payload_coded = self._mbproto.create_modbus_periodic(self._modbus_cfg_idx, self._modbus_interval,
                                                                               self._payload_coded)
            else:
                raise ValueError("No Modbus Payload!")
        else:
            raise ValueError("Unsupported command type!")

    def _stop_sinks(self):
        self._client.deinitialize_sink()

    def _start_sinks(self):
        self._client.initialize_sink()

    def send_command(self):
        if self._client and self._payload_coded:
            self._client.send(self._payload_coded)

    def initialize_sink(self):
        self._start_sinks()

    def deinitialize_sink(self):
        self._stop_sinks()

    async def run(self, quit=False):
        if self._cmd_type is not None:
            self.send_command()
        if not quit:
            logging.info("Entering infinite polling, press Ctrl+C to exit")
        while True:
            response = await self._client.async_receive()
            logging.info(f"Got message from: {response.src}")
            self._mbproto.print_decoded_msg(response.payload)
            if quit:
                return

    async def run_periodically(self, period=10, timeout=5, _callback=None, callback_args=None, print_default=False):
        if self._cmd_type is None:
            raise ValueError("Command not defined!")
        logging.info("Entering periodical command send with polling, press Ctrl+C to exit")
        while True:
            self.send_command()
            try:
                response = await asyncio.wait_for(self._client.async_receive(), timeout=timeout)
            except asyncio.TimeoutError:
                logging.warning("No response from the device!")
                await asyncio.sleep(period)
                continue
            if _callback != None:
                ret, err, msg = self._mbproto.decode_response(response.payload)
                if (not ret):
                    logging.error("Failed to decode frame!: %s", err)
                else:
                    _callback(msg, callback_args)
            elif print_default:
                logging.info(f"Got message from: {response.src}")
                self._mbproto.print_decoded_msg(response.payload, decode_modbus_frame=False)
            await asyncio.sleep(period)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: mb_protocol_enums.proto
# Protobuf Python Version: 5.28.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    2,
    '',
    'mb_protocol_enums.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17mb_protocol_enums.proto\x12\x15wmb.proto.mb_protocol*V\n\nModbusMode\x12\x17\n\x13MODBUS_MODE_UNKNOWN\x10\x00\x12\x16\n\x12MODBUS_MODE_MASTER\x10\x01\x12\x17\n\x13MODBUS_MODE_SNIFFER\x10\x02*R\n\x0f\x41ntennaSettings\x12\x13\n\x0f\x41NTENNA_UNKNOWN\x10\x00\x12\x14\n\x10\x41NTENNA_INTERNAL\x10\x01\x12\x14\n\x10\x41NTENNA_EXTERNAL\x10\x02*P\n\nModbusPort\x12\x17\n\x13MODBUS_PORT_UNKNOWN\x10\x00\x12\x14\n\x10MODBUS_PORT_ZERO\x10\x01\x12\x13\n\x0fMODBUS_PORT_ONE\x10\x02*\xc8\x01\n\x08PortBaud\x12\x15\n\x11PORT_BAUD_UNKNOWN\x10\x00\x12\x12\n\x0ePORT_BAUD_4800\x10\x0c\x12\x12\n\x0ePORT_BAUD_9600\x10\r\x12\x13\n\x0fPORT_BAUD_19200\x10\x0e\x12\x13\n\x0fPORT_BAUD_28800\x10\x0f\x12\x13\n\x0fPORT_BAUD_38400\x10\x10\x12\x13\n\x0fPORT_BAUD_57600\x10\x11\x12\x13\n\x0fPORT_BAUD_76800\x10\x12\x12\x14\n\x10PORT_BAUD_115200\x10\x13*\x93\x01\n\nPortParity\x12\x17\n\x13PORT_PARITY_UNKNOWN\x10\x00\x12\x14\n\x10PORT_PARITY_NONE\x10\x01\x12\x13\n\x0fPORT_PARITY_ODD\x10\x02\x12\x14\n\x10PORT_PARITY_EVEN\x10\x03\x12\x14\n\x10PORT_PARITY_MARK\x10\x04\x12\x15\n\x11PORT_PARITY_SPACE\x10\x05*\x86\x01\n\x0cPortStopBits\x12\x1a\n\x16PORT_STOP_BITS_UNKNOWN\x10\x00\x12\x16\n\x12PORT_STOP_BITS_0_5\x10\x01\x12\x14\n\x10PORT_STOP_BITS_1\x10\x02\x12\x16\n\x12PORT_STOP_BITS_1_5\x10\x03\x12\x14\n\x10PORT_STOP_BITS_2\x10\x04\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'mb_protocol_enums_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_MODBUSMODE']._serialized_start=50
  _globals['_MODBUSMODE']._serialized_end=136
  _globals['_ANTENNASETTINGS']._serialized_start=138
  _globals['_ANTENNASETTINGS']._serialized_end=220
  _globals['_MODBUSPORT']._serialized_start=222
  _globals['_MODBUSPORT']._serialized_end=302
  _globals['_PORTBAUD']._serialized_start=305
  _globals['_PORTBAUD']._serialized_end=505
  _globals['_PORTPARITY']._serialized_start=508
  _globals['_PORTPARITY']._serialized_end=655
  _globals['_PORTSTOPBITS']._serialized_start=658
  _globals['_PORTSTOPBITS']._serialized_end=792
# @@protoc_insertion_point(module_scope)



import argparse
import asyncio
from wmbc.wmbc import WMBController

async def main():
    """
        Main for WMB Controller
    """

    parser = argparse.ArgumentParser(description='WMB Controller - sends out MB Protocol requests over Wirepas \
            (assumes the Sink is preconfigured with correct Network Address and Channel).')
    parser.add_argument(
        '--dst-addr',
        required=False,
        type=int,
        default=21,
        help='Destination address'
    )
    parser.add_argument(
        '--cmd',
        required=False,
        type=str,
        choices=['reset', 'diag', 'dev_mode', 'ant_cfg', 'port_cfg', 'modbus_1s', 'modbus_p'],
        help='Command Types'
    )
    parser.add_argument(
            '--dev-mode',
            required=False,
            type=int,
            choices=[0, 1],
            help='Device mode: 0 - Modbus Master, 1 - Modbus Sniffer'
    )
    parser.add_argument(
            '--ant-cfg',
            required=False,
            type=int,
            choices=[0, 1],
            help='Antenna config: 0 - Internal Antenna, 1 - External Antenna'
    )
    parser.add_argument(
            '--target-port',
            required=False,
            type=int,
            choices=[1, 2],
            help='Target port: 1 - Port 1 (Channel 1), 2 - Port 2 (Channel 2); Use it with port-cfg or modbus commands'
    )
    parser.add_argument(
            '--port-cfg',
            required=False,
            nargs=3,
            help='Port Serial configuration: <baudrate> <parity (0 - none, 1 - odd, 2 - even)> <stop_bits (1 or 2)>'
    )
    parser.add_argument(
            '--modbus-file',
            required=False,
            type=str,
            help='File with binary modbus frame needed for modbus_1s or modbus_p commands'
    )
    parser.add_argument(
            '--modbus-cfg-idx',
            required=False,
            type=int,
            help='Config index for periodic modbus frame'
    )
    parser.add_argument(
            '--modbus-interval',
            required=False,
            type=int,
            help='Interval for periodic modbus frame'
    )
    args = parser.parse_args()
    args_dict = vars(args)

    WMBC = WMBController(**args_dict)
    WMBC.initialize_sink()
    await WMBC.run()

if __name__ == "__main__":
    asyncio.run(main())
